import { TestResponse } from '../src/TestResponse';
import files from '../src/Filesystem';
import { fixturePath, projectPath } from './helpers';
import { Diagnostic, DiagnosticSeverity } from 'vscode-languageserver-types';

describe('TestResponse', () => {
    let testResponse: TestResponse;

    beforeAll(async () => {
        const output = await files.get(fixturePath('test-result.txt'));
        testResponse = new TestResponse(output);
    });

    describe('Diagnostic', () => {
        let diagnosticGroup: Map<string, Diagnostic[]>;

        const expectedDiagnostic = () => {
            return jasmine.objectContaining({
                message: jasmine.any(String),
                range: {
                    end: {
                        character: jasmine.any(Number),
                        line: jasmine.any(Number),
                    },
                    start: {
                        character: jasmine.any(Number),
                        line: jasmine.any(Number),
                    },
                },
                relatedInformation: jasmine.anything(),
                severity: DiagnosticSeverity.Error,
                source: 'PHPUnit',
            });
        };

        beforeAll(async () => {
            diagnosticGroup = await testResponse.asDiagnosticGroup();
        });

        it('Recca0120\\VSCode\\Tests\\CalculatorTest', () => {
            const diagnostics: Diagnostic[] = diagnosticGroup.get(
                projectPath('tests/CalculatorTest.php').toString()
            );

            expect(diagnostics).toEqual([
                expectedDiagnostic(),
                expectedDiagnostic(),
            ]);
        });

        it('Recca0120\\VSCode\\Tests\\AssertionsTest', () => {
            const diagnostics: Diagnostic[] = diagnosticGroup.get(
                projectPath('tests/AssertionsTest.php').toString()
            );

            expect(diagnostics).toEqual([
                expectedDiagnostic(),
                expectedDiagnostic(),
                expectedDiagnostic(),
                expectedDiagnostic(),
                expectedDiagnostic(),
                expectedDiagnostic(),
            ]);
        });
    });
});
